/* packet-moldudp.c
 * Routines for Nasdaq MoldUDP dissection
 * Copyright 2010, Chris Bidwell <chris (at) chrisbidwell.com>
 *
 * 0.1.0.0
 *
 * Wireshark - Network traffic analyzer
 * By Gerald Combs <gerald@wireshark.org>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif
#include <intsafe.h>
#include <epan/packet.h>

/* There is no specific port for MoldUDP, but this one was used in a stream I dissected */
#define MOLDUDP_PORT 31041

static int proto_moldudp = -1;

static int hf_moldudp_pdu_type = -1;

static gint ett_moldudp = -1;

static int hf_moldudp_session_number = -1;
static int hf_moldudp_sequence_number = -1;
static int hf_moldudp_message_count = -1;
static int hf_moldudp_next_sequence_number = -1;



void
proto_register_moldudp(void)
{

	static hf_register_info hf[] = {
		{ &hf_moldudp_session_number,
			{ "Session Number", "moldudp.session_number",
			FT_STRING, BASE_NONE,
			NULL, 0x0,
			NULL, HFILL }
		},
		{ &hf_moldudp_sequence_number,
			{ "Sequence Number", "moldudp.sequence_number",
			FT_UINT32, BASE_DEC,
			NULL, 0X0,
			NULL, HFILL }
		},
		{ &hf_moldudp_message_count,
			{ "Message Count", "moldudp.message_count",
			FT_UINT16, BASE_DEC,
			NULL, 0X0,
			NULL, HFILL }
		},
		{ &hf_moldudp_next_sequence_number,
			{ "Next Sequence Number", "moldudp.next_sequence_number",
			FT_UINT32, BASE_DEC,
			NULL, 0X0,
			NULL, HFILL }
		}


	};

	/* Setup protocol subtree array */
	static gint *ett[] = {
		&ett_moldudp
	};

	proto_moldudp = proto_register_protocol (
		"Mold UDP Protocol",	/* name       */
		"MoldUDP",		/* short name */
		"mold"		/* abbrev     */
		);

	proto_register_field_array(proto_moldudp, hf, array_length(hf));
	proto_register_subtree_array(ett, array_length(ett));
}

static void
dissect_moldudp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
{
	gint offset = 0;
	gint i = 0;
	guint16 message_count = 0;
	guint32 curr_sequence = 0;
	guint packet_length = tvb_length_remaining(tvb, offset);

	/*	Should really check that the payload contains enough data to be a genuine MOLD UDP message.
		Headers are 16bytes.
		Function definition is incompatible with a return type of int. Left disabled for the moment. */
/*
	if (packet_length < 16)
		return 0;
*/
	/* Could instead wrap this block in 'if (packet_length >= 16)', but it doesn't do any harm as it is. */
	
	col_set_str(pinfo->cinfo, COL_PROTOCOL, "MOLDUDP");

	/* TODO: Make this something useful in the future. */
	/* Clear out stuff in the info column. */
	col_clear(pinfo->cinfo,COL_INFO);

	message_count = tvb_get_letohs(tvb, 14);
	if (message_count==(guint16)0) /* message_count == 0, HEARTBEAT MESSAGE */
	{
		col_set_str(pinfo->cinfo, COL_INFO, "Heartbeat");
	}

	if (tree) { /* we are being asked for details */
		proto_item *ti = NULL;
		proto_tree *moldudp_tree = NULL;
		
		ti = proto_tree_add_item(tree, proto_moldudp, tvb, 0, -1, FALSE);
		moldudp_tree = proto_item_add_subtree(ti, ett_moldudp);
		
		/* These fields are defined in the protocol documentation at */
		/* http://www.nasdaqtrader.com/content/technicalsupport/specifications/dataproducts/moldudp.pdf */
		proto_tree_add_item(moldudp_tree, hf_moldudp_session_number, tvb, offset, 10, TRUE); offset += 10;
		curr_sequence = tvb_get_letohl(tvb,10);
		proto_tree_add_item(moldudp_tree, hf_moldudp_sequence_number, tvb, offset, 4, TRUE); offset += 4;
		proto_tree_add_item(moldudp_tree, hf_moldudp_message_count, tvb, offset, 2, TRUE); offset += 2;
		proto_tree_add_uint(moldudp_tree, hf_moldudp_next_sequence_number, tvb, 10, 4,curr_sequence+(guint32)message_count); 
		
		if (message_count > 0)
		{
		/* Amusingly, this code caused the frame description pane to spit out an error when decoding. */
		/* Haven't had the patience to debug it yet. */
/*		
			while (message_count>0)
			{
				i = tvb_get_ntohs(tvb,offset);
				offset += 2;
				if (i>0)
				{
					proto_tree_add_item(moldudp_tree, hf_moldudp_message_block, tvb, offset, i, FALSE);
					offset += i;
				}
				else
				{
					proto_tree_add_item(moldudp_tree, hf_moldudp_message_block, "Session Termination", 0, -1, TRUE);
				}
				message_count=0;
			}
*/
			/* TODO: Work out if this is a downstream packet or a request packet. &pinfo->dst should be multicast if downstream */
			
			/* TODO: Add new substrees for message blocks. message_count contains the number of blocks. */
		}
	}
	
	/*	Another dissector developer mentioned returning the length of the payload you'd processed
		but this caused compile errors. Maybe just changing static void to static int in the
		function declaration would suffice. */
	/* return tvb_length(tvb); */
}

void
proto_reg_handoff_moldudp(void)
{
	static dissector_handle_t moldudp_handle;

	moldudp_handle = create_dissector_handle(dissect_moldudp, proto_moldudp);
	dissector_add("udp.port", MOLDUDP_PORT, moldudp_handle);
}

